package com.mortals.xhx.module.capture.service;

import com.sun.jna.Library;

/**
 * 高拍仪so dell库
 *
 * @author: ZYW
 * @date: 2024/01/24 09:59
 */
public interface CaptureSDK extends Library {

    static int SXT_OK = 0;   //成功
    static int SXT_NOT_INIT_ERROR = -1;   //SDK未初始化
    static int SXT_NOT_DEVICE_ERROR = -2;   //设备不存在
    static int SXT_OPEN_DEVICE_ERROR = -3;   //设备打开失败
    static int SXT_PHOTOGRAPH_ERROR = -4;   //拍照失败，摄像头未打开
    static int SXT_RESOLUTION_ERROR = -5;   //分辨率切换失败
    static int SXT_MEMORY_ERROR = -6;   //传入空间不够
    static int SXT_FILE_ERROR = -7;   //文件打开失败
    static int SXT_FILE_DELETE_ERROR = -8;   //文件删除失败
    static int SXT_BARCODE_ERROR = -9;   //条码识别失败
    static int SXT_THREAD_ERROR = -10;   //高拍仪线程初始化失败
    static int SXT_ERROR_1 = -11;   //参数错误
    static int SXT_ERROR_2 = -12;   //文件不存在
    static int SXT_ERROR_3 = -13;   //不支持的指令
    static int SXT_ERROR_4 = -14;   //摄像头数据读取出错
    static int SXT_ERROR_5 = -15;   //图片转PDF失败
    static int SXT_ERROR_6 = -16;   //身份证信息获取失败
    static int SXT_ERROR_7 = -17;   //没有这个功能


    /*********************************
     * 功能:
     *     初始化高拍仪(必须先调用)
     * 参数:
     *      无
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_Init();
    /*********************************
     * 功能:
     *     释放高拍仪(关闭高拍仪时必须调用)
     * 参数:
     *      无
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_Release();
    /*********************************
     * 功能:
     *     视频流回调
     * 参数:
     *      imgData       -       图片二进制数据
     *      len             -     图片长度
     * 返回值:
     *      参见: 错误代码
     **********************************/
    void DoSXT_VideoStreaming_Callback(String imgData, int len);
    /*********************************
     * 功能:
     *     获取摄像头名称列表
     * 参数:
     *      无
     * 返回值:
     *      参见: 返回json字符串[{"index":1,"name":"video1"},
     *                         {"index":2,"name":"video2"}]
     *           如果没有找到返回空
     **********************************/
    String SXT_GetVideoInfos();
    /*********************************
     * 功能:
     *     打开主头(副头打开时自动关闭副头)
     * 参数:
     *      width       -       分辨率宽度
     *      height      -       分辨率高度
     * 返回值:
     *      参见: 错误代码
     **********************************/
//    int SXT_OpenHostDevice(int width, int height, DoSXT_VideoStreaming_Callback callback);
    /*********************************
     * 功能:
     *     打开副头(主头打开时自动关闭主头)
     * 参数:
     *      width       -       分辨率宽度
     *      height      -       分辨率高度
     * 返回值:
     *      参见: 错误代码
     **********************************/
//    int SXT_OpenDeputyDevice(int width,int height, DoSXT_VideoStreaming_Callback callback);
    /*********************************
     * 功能:
     *     关闭摄像头
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_CloseDevice();

    /*********************************
     * 功能:
     *     获取指定摄像头分辨率json列表
     * 参数:
     *      index       -       1 主头 2 副头(不是这个两个直接报错)
     * 返回值:
     *      参见: 返回json字符串 [{"index":1,"name":"640x480"},
     *                          {"index":2,"name":"800x600"}]
     *           如果没有找到返回空
     **********************************/
    String SXT_GetResolution(int index);

    /*********************************
     * 功能:
     *     是否裁边拍照(只支持主头, 副头没有功能, 默认开启)
     * 参数:
     *      type       -       0 不裁边 1 裁边
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_SetCutType(int type);

    /*********************************
     * 功能:
     *     设置视频画面模式
     * 参数:
     *      type       -       设置颜色 1 彩色 2 灰度 3 黑白
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_SetColorType(int type);
    /*********************************
     * 功能:
     *     视频画面旋转
     * 参数:
     *      direction    -      旋转方向(左边 0 右边 1)
     *      rotate       -      旋转角度(默认90)
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_SetRotate(int direction, int rotate);
    /*********************************
     * 功能:
     *     拍照
     * 参数:
     *      index    -       1 文件拍照 2 base64 拍照
     *      path     -      文件路径(可选)
     * 回调：
     *      index   -   拍照方式 1 文件拍照 2 base64 拍照
     *      imgData -   文件拍照或拍照失败时为空字符串
     *      len     -   文件拍照或拍照失败时为-1
     *      isOk    -   0 拍照成功 其他拍照失败
     *      isDelete -  如果是pdf会删除原图片此时返回1，否则返回0(默认)
     * 返回值:
     *      参见: 错误代码
     **********************************/
    void DoSXT_Photograph_Callback(int index, String imgData, int len, int isOk, int isDelete);
//    int SXT_Photograph(int index, String path, DoSXT_Photograph_Callback callback);

    /*********************************
     * 功能:
     *     设置分辨率
     * 参数:
     *      resolutionIndex    -       分辨率编号 0开始
     * 返回值:
     *      参见: 错误代码
     **********************************/
    int SXT_SetResolution(int resolutionIndex);
    int SXT_SetResolution2(int w,int h);
    /*********************************
     * 功能:
     *     图形拆分
     * 参数:
     *      imgPath     -       需要拆分的文件绝对路径
     *      dir         -       保存文件夹
     *      type        -       1 左右 2 上下
     * 返回值:
     *      成功返回json字符串， 失败返回空字符串 {"path1":"/tmp/1.jpg","path1":"/tmp/2.jpg"}
     **********************************/
    String SXT_SplitImage(String imgPath, String dir, int type);
    /*********************************
     * 功能:
     *     图形合并
     * 参数:
     *      imgPath1     -       需要合并的文件1绝对路径
     *      imgPath2     -       需要合并的文件2绝对路径
     *      path         -       保存路径
     *      type        -        1 左右 2 上下
     * 返回值:
     *      成功返回json字符串， 失败返回空字符串 {"path":"/tmp/1.jpg"}
     **********************************/
    String SXT_CombineImage(String imgPath1, String imgPath2, String savePath, int type);

    /*********************************
     * 功能:
     *     文件转Base64
     * 参数:
     *     file    -       文件路径
     * 返回值:
     *      参见: 成功返回base64数据 失败返回 空
     **********************************/
    String SXT_FileToBase64(String path);


    /*********************************
     * 功能:
     *     一维码识别
     * 参数:
     *     file    -       一维码图片
     * 返回值:
     *      参见: 成功返回扫码到的数据 失败返回 空
     **********************************/
    String SXT_DecodeBarcodeImage(String path);

}