package com.mortals.xhx.module.capture.service.impl;

import com.alibaba.fastjson.JSON;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.util.DataUtil;
import com.mortals.xhx.busiz.req.PrintReq;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.capture.model.File;
import com.mortals.xhx.module.capture.service.CaptureService;
import com.mortals.xhx.module.print.model.Printer;
import com.mortals.xhx.module.print.model.PrinterQueue;
import com.mortals.xhx.module.print.service.PrintComponent.BasePrintComponent;
import com.mortals.xhx.module.print.service.PrintComponent.ComponentCons;
import com.mortals.xhx.module.print.service.PrintService;
import com.mortals.xhx.opencv.UI.CameraOprationByOpenCV;
import com.mortals.xhx.opencv.UI.CaptureUI;
import com.mortals.xhx.opencv.bean.PhotoBean;
import com.mortals.xhx.swing.SwingArea;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import javax.print.attribute.Attribute;
import javax.print.attribute.AttributeSet;
import java.awt.print.PrinterJob;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 高拍仪实现类
 *
 * @author: zxfei
 * @date: 2024/1/2 10:00
 */
@Service
@Slf4j
@ConditionalOnProperty(prefix = "config", name = "capture", havingValue = "default")
public class CaptureServiceImpl implements CaptureService {

    @Value("${siteconfig.configpath:'/root/config.prop'}")
    String configPath;
    @Value("${active.active:'reg'}")
    String active;
    @Value("${config.captureUrl:ws://localhost:1818/}")
    private String url;

    @Override
    public Rest<List<File>> getCapture(String waitTime) {
        Rest<ArrayList<PhotoBean>> rest = null;
        log.info("active:{}",active);
        if(active.equals("reg")){ //若尔盖是老版本opencv打开高拍仪
            rest = CameraOprationByOpenCV.getInstance().initWindow(DataUtil.converStr2Int(waitTime, 120),configPath);
        }else {  //后续对接用汉腾提供的api开发
            rest = CaptureUI.getInstance().initWindow(DataUtil.converStr2Int(waitTime, 120),configPath,url);
        }

        if (rest.getCode() == YesNoEnum.YES.getValue()) {
            log.info("fileData:{}", JSON.toJSONString(rest.getData()));
            List<File> fileList = rest.getData().stream().map(item -> {
                File file = new File();
                file.setFilename(item.getFilename());
                file.setBase64String(item.getBase64String());
                return file;
            }).collect(Collectors.toList());
            return Rest.ok("获取拍照列表成功!", fileList);
        } else {
            return Rest.fail(rest.getMsg());
        }
    }


    public static void main(String[] args) {
        PhotoBean photoBean = new PhotoBean("111","222");
        System.out.println(JSON.toJSONString(photoBean));
    }
}
