package com.mortals.xhx.module.sign.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.xhx.busiz.req.SignReq;
import com.mortals.xhx.busiz.rsp.SignResp;
import com.mortals.xhx.module.sign.service.SignService;
import lombok.extern.slf4j.Slf4j;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.net.URI;
import java.net.URISyntaxException;


/**
 * 签名版实现类
 *
 * @author: zhou
 * @date: 2024/7/8 10:00
 */
@Service
@Slf4j
public class SignServiceImpl implements SignService {

    @Value("${config.signUrl:ws://127.0.0.1:29999/}")
    private String url;

    static int runTime = 60;   //60S超时返回

    @Override
    public SignResp getSign(String signType) {
        WebSocketClient webSocketClient = null;

        URI uri;

        SignResp signResp = new SignResp();

        if(runTime != 60){
            signResp.setHWPenSign(signType);
            signResp.setMsgID(-1);
            signResp.setMessage("操作中，请勿重复发送指令");
            return signResp;
        }

        //提交操作参数
        SignReq signReq = new SignReq();
        signReq.setHWPenSign(signType);


        try {
            uri = new URI(url);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        if(webSocketClient != null && !webSocketClient.isClosed()){
            String msg = JSON.toJSONString(signReq);
            log.info("发送msg = " +msg);
            webSocketClient.send(msg.getBytes());
        }else {
            webSocketClient = new WebSocketClient(uri) {
                @Override
                public void onOpen(ServerHandshake handshakedata) {
                    String msg = JSON.toJSONString(signReq);
                    log.info("新连接已打开:msg =" +msg);
                    send(msg.getBytes());
                }

                @Override
                public void onMessage(String message) {
                    log.info("接收到消息: {}",message);

                    if(message != null){
                        JSONObject jsonObject = JSON.parseObject(message);
                        signResp.setHWPenSign(jsonObject.getString("HWPenSign"));
                        signResp.setMsgID(jsonObject.getInteger("msgID"));
                        signResp.setMessage(jsonObject.getString("message"));
                    }
                }

                @Override
                public void onClose(int code, String reason, boolean remote) {
                    log.info("连接已关闭,code:{} reason:{} {}",code,reason,remote);
                    signResp.setHWPenSign(signType);
                    signResp.setMsgID(-1);
                    signResp.setMessage("连接已关闭,code="+code+",reason="+reason);
                }

                @Override
                public void onError(Exception ex) {
                    ex.printStackTrace();
                    log.info("链接出错了"+ex.toString());
                    signResp.setHWPenSign(signType);
                    signResp.setMsgID(-1);
                    signResp.setMessage(ex.toString());
                }
            };
            webSocketClient.connect(); // 连接到WebSocket服务器
        }

        while (true){
            try {
                runTime --;
                Thread.sleep(1000);
            } catch (InterruptedException e) {

            }

            if(signResp.getHWPenSign() != null && !signResp.getHWPenSign().equals("")){
                runTime = 60;
                return signResp;
            }

            if(runTime <= 0){
                runTime = 60;
                signResp.setHWPenSign(signType);
                signResp.setMsgID(-1);
                signResp.setMessage("链接签名板失败，操作超时");
                return signResp;
            }
        }

    }

    public static void main(String[] args) {
        URI uri = null;
        try {
            SignReq signReq = new SignReq();
            signReq.setHWPenSign("HWGetDeviceStatus");
            uri = new URI("ws://192.168.0.158:29999/");

            WebSocketClient webSocketClient = new WebSocketClient(uri) {
                @Override
                public void onOpen(ServerHandshake handshakedata) {
                    System.out.println("新连接已打开");
                }

                @Override
                public void onMessage(String message) {
                    System.out.println("接收到消息: "+message);


                }

                @Override
                public void onClose(int code, String reason, boolean remote) {
                    System.out.println("连接已关闭,code="+code+",reason="+reason);
                }

                @Override
                public void onError(Exception ex) {
                    ex.printStackTrace();
                    System.out.println("链接出错了"+ex.toString());
                }
            };

            webSocketClient.connect(); // 连接到WebSocket服务器
            webSocketClient.send(JSON.toJSONString(signReq));

        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }

    }


}
