package com.mortals.xhx.module.social.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.xhx.busiz.req.SignReq;
import com.mortals.xhx.busiz.req.SocialReq;
import com.mortals.xhx.busiz.rsp.SignResp;
import com.mortals.xhx.busiz.rsp.SocialResp;
import com.mortals.xhx.module.social.service.SocialService;
import lombok.extern.slf4j.Slf4j;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.net.URI;
import java.net.URISyntaxException;


/**
 * 社保卡实现类
 *
 * @author: zhou
 * @date: 2024/7/8 10:00
 */
@Service
@Slf4j
public class SocialServiceImpl implements SocialService {

    @Value("${config.socialUrl:ws://localhost:12342/}")
    private String url;

    static int runTime = 60;   //60S超时返回

    WebSocketClient webSocketClient = null;

    SocialResp socialResp = null;

    @Override
    public SocialResp getSocial(String socialType) {

        URI uri;


        socialResp = new SocialResp();
        socialResp.setMessage("");

        //提交操作参数
        SocialReq socialReq = new SocialReq();
        socialReq.setFunction(socialType);
        if(socialType.equals("iReadSFZ")){  //读身份证
            socialReq.setPchPhotoAddr("/tmp/head.bmp");
        }

        if(socialType.equals("iReadCardBas_HSM_Step1")){  //社保卡识别类型   1接触卡  2非接触卡
            socialReq.setIType("2");
        }

        try {
            uri = new URI(url);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        if(webSocketClient != null){
            String msg = JSON.toJSONString(socialReq);
            log.info("发送msg = " +msg);
            webSocketClient.send(msg);
        }else {
            webSocketClient = new WebSocketClient(uri) {
                @Override
                public void onOpen(ServerHandshake handshakedata) {
                    String msg = JSON.toJSONString(socialReq);
                    log.info("新连接已打开:msg =" +msg);
                    send(msg);
                }

                @Override
                public void onMessage(String message) {
                    log.info("接收到消息: {}",message);

                    if(message != null){
                        socialResp.setMessage(message);
                    }
                }

                @Override
                public void onClose(int code, String reason, boolean remote) {
                    log.info("连接已关闭,code:{} reason:{} {}",code,reason,remote);
                    socialResp.setMessage("连接已关闭,code="+code+",reason="+reason);
                }

                @Override
                public void onError(Exception ex) {
                    ex.printStackTrace();
                    log.info("链接出错了"+ex.toString());
                    socialResp.setMessage(ex.toString());
                }
            };
            webSocketClient.connect(); // 连接到WebSocket服务器
        }

        while (true){
//            log.info("runtime:{}",runTime);
            try {
                runTime --;
                Thread.sleep(1000);
            } catch (InterruptedException e) {

            }

            if(socialResp.getMessage() != null && !socialResp.getMessage().equals("")){
                runTime = 60;
                break;
            }

            if(runTime <= 0){
                runTime = 60;
                socialResp.setMessage("链接社保卡失败，操作超时");
                break;
            }
        }

        return socialResp;

    }


}
