package com.mortals.xhx.opencv;


import lombok.extern.slf4j.Slf4j;
import org.bytedeco.javacpp.Pointer;
import org.bytedeco.javacv.Java2DFrameUtils;
import org.bytedeco.opencv.opencv_core.Mat;
import org.bytedeco.opencv.opencv_core.MatVector;
import org.bytedeco.opencv.opencv_core.Rect;
import org.opencv.imgproc.Imgproc;


import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.util.ArrayList;
import java.util.List;

import static org.bytedeco.opencv.global.opencv_highgui.*;//包含了所有图形接口函数
import static org.bytedeco.opencv.global.opencv_imgcodecs.*;
import static org.bytedeco.opencv.global.opencv_imgproc.*; //COLOR_RGB2GRAY

@Slf4j
public class Demo {

    public  void test(){
        String filePath = "F:\\opencv-demo\\1704165443969.jpg";
        Mat sourceImage = imread(filePath); // 加载图像

     /*   Imgproc.cvtColor(sourceImage, gray, Imgproc.COLOR_BGR2GRAY);*/

        Mat gray = new Mat();


        BufferedImage bufferedImage = Java2DFrameUtils.toBufferedImage(sourceImage);
        Mat mat = Java2DFrameUtils.toMat(bufferedImage);

        // new OpenCVFrameConverter.ToMat()

        cvtColor(sourceImage,  gray, COLOR_BGR2GRAY); // 转换颜色从BGR到RGB

        Mat binary = new Mat();
        threshold(gray, binary, 0, 255, THRESH_BINARY | THRESH_OTSU);
        
        Mat edges = new Mat();
        Canny(binary, edges, 50, 150);

        Mat hierarchy = new Mat();
        MatVector matVector = new MatVector();

        findContours(binary,matVector,hierarchy,RETR_TREE, CHAIN_APPROX_SIMPLE);

        // 找出最大的轮廓（假设这是图片内容）
        double maxArea = 0;
        int maxContourIndex = 0;
        for (int i = 0; i < matVector.size(); i++) {
            double area = contourArea(matVector.get(i));
            if (area > maxArea) {
                maxArea = area;
                maxContourIndex = i;
            }
        }

        Rect rect = boundingRect(matVector.get(maxContourIndex));

        Mat outImage = new Mat(sourceImage, rect);

        imwrite("F:\\opencv-demo\\outimage.jpg", outImage);

      //  BufferedImage bufferedImage = matToBufferedImage(outImage);




//        List<Pointer> contours = new ArrayList<>();
//        Mat hierarchy = new Mat();
   //     findContours(binary, contours, hierarchy,RETR_TREE, CHAIN_APPROX_SIMPLE);

        log.info("image rows:{}",sourceImage.rows());



        // 转为灰度图像




/*        imshow("1", image);// 原始图像
        Mat gray = new Mat();
        cvtColor(image, gray, COLOR_BGRA2GRAY); // 彩色图像转为灰度图像COLOR_RGB2GRAY
        imshow("2", gray);// 灰度图像
        Mat bin = new Mat();
        threshold(gray, bin, 120, 255, THRESH_TOZERO); // 图像二值化
        imshow("3", bin);// 二值图像
        waitKey(0);*/
    }

    public static BufferedImage matToBufferedImage(Mat mat) {
        byte[] data = new byte[mat.channels() * mat.cols() * mat.rows()];
        //mat.get(0, 0, data); // 获取所有像素值
        BufferedImage image = new BufferedImage(mat.cols(), mat.rows(), mat.channels() == 1 ? BufferedImage.TYPE_BYTE_GRAY : BufferedImage.TYPE_3BYTE_BGR);
        final byte[] targetPixels = ((DataBufferByte) image.getRaster().getDataBuffer()).getData();
        System.arraycopy(data, 0, targetPixels, 0, data.length);
        return image;
    }


    public static void main(String[] args) {
       new Demo().test();
    }
}
