package com.mortals.xhx.opencv.UI;


import com.google.gson.Gson;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.opencv.bean.ConfigBean;
import com.mortals.xhx.opencv.bean.PhotoBean;
import com.mortals.xhx.opencv.utils.ImageUtil;
import com.mortals.xhx.opencv.utils.OpencvHelper;
import lombok.extern.slf4j.Slf4j;
import org.bytedeco.opencv.opencv_core.Mat;
import org.bytedeco.opencv.opencv_videoio.VideoCapture;

import org.springframework.util.ObjectUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

import static org.bytedeco.opencv.global.opencv_videoio.CAP_PROP_FRAME_HEIGHT;
import static org.bytedeco.opencv.global.opencv_videoio.CAP_PROP_FRAME_WIDTH;


/**
 * @author ZYW
 * @date 2024-01-08 13:55
 */
@Slf4j
public class CameraOprationByOpenCV {


    static String configPath = "F:\\\\config.prop";  //默认测试本机路径

    private static CameraOprationByOpenCV instance = null;
    static JFrame frame;

    static VideoCapture capture;  //摄像头视频流

    static Dialog setDialog;   //配置框
    static Dialog preDialog;   //预览照片dialog

    protected static VideoPanel videoCamera = new VideoPanel();

    public static ArrayList<PhotoBean> photoBeanArrayList = new ArrayList<>();

    static Gson gson = new Gson();
    public static int cutType = 0;  //0不切边  1自动切边  2 自定义切边
    public static int roteAngle = 0;   //旋转角度
    public static int cameraIndex = 0;   //选中的摄像头
    public static String cameraName = "";   //选中的摄像头名称
    public static int maxPhoto = 0;     //照片最大拍照张数  0为不限制
    public static String ratio = "640x480";     //摄像头默认分辨率

    static int timer = 300;   //用于两分钟无操作计时

    static int preHeight = 480;  //预览界面的高度

    static Mat sourceMat;  //原始图像

    static Mat outMat; //缩放后的图像

    static int count = 0;

    public static void main(String[] args) {
        //初始化窗口
        initWindow(300,configPath);
    }



    private CameraOprationByOpenCV() {
    }

    public static CameraOprationByOpenCV getInstance() {
        if (null == instance) {
            synchronized (CameraOprationByOpenCV.class) {
                if (null == instance) {
                    instance = new CameraOprationByOpenCV();
                }
            }
        }

        return instance;
    }

    /**
     * 读取本地文件配置
     */
    private static void redConfig(){
        File file = new File(configPath);
        try {
            file.createNewFile();
        } catch (IOException e) {
            log.error( "创建文件失败" ); ;
        }

        Scanner sc = null ;
        try
        {
            sc = new Scanner( file ) ;

            String jsonString = sc.next();
            ConfigBean jo = gson.fromJson(jsonString,ConfigBean.class);


            if(jo != null){
                cutType = jo.getCutType();
                roteAngle = jo.getRoteAngle();
                cameraIndex = jo.getCameraIndex();
                maxPhoto = jo.getMaxPhoto();
                if(jo.getRatio() != null){
                    ratio = jo.getRatio();
                }
            }else {
                log.info( "配置文件为空" ); ;
            }


        }
        catch( Exception e )
        {
            log.error( "读取配置文件失败"+e.toString() ); ;
        }
        finally
        {
            try
            {
//                log.error( "读写关闭" ) ;
                sc.close();
            }
            catch ( Exception e )
            {

            }
        }
    }

    /**
     * 修改本地文件配置
     */
    private static void writeConfig(){

        ConfigBean cb = new ConfigBean(cutType,roteAngle,cameraIndex,maxPhoto,ratio);

        File file = new File(configPath);
        try {
            file.createNewFile();
        } catch (IOException e) {
        }

        FileWriter fileWriter = null ;
        BufferedWriter bufferedWriter = null ;  //FileWriter和BufferedWriter的初始化需要监听，此处只定义不初始化

        try
        {
            fileWriter = new FileWriter( file ) ;
            bufferedWriter = new BufferedWriter( fileWriter ) ;

            bufferedWriter.write( gson.toJson(cb) ) ;
        }
        catch( Exception e )
        {
            log.error( "Error." ); ;
        }
        finally
        {
            try
            {

                if(bufferedWriter != null){
                    bufferedWriter.close(); //随手关闭文件,此处注意关闭顺序不能错误
                }
                if(fileWriter != null){
                    fileWriter.close();
                }
            }
            catch( IOException ioe )
            {

            }

        }

    }

    /**
     * 初始化窗口
     */
    public static Rest<ArrayList<PhotoBean>> initWindow(int runtime,String path){
       configPath = path;
        photoBeanArrayList.clear();
        redConfig();
        timer = runtime;
        //AWT中接口LayoutManager有五个实现类：GridLayout（网格布局）、FlowLayout（流式布局）、CardLayout（卡片布局）、
        // GridBagLayout（网格包布局）和BorderLayout（边框布局）。为了简化开发，Swing引入了一个新的布局管理器BoxLayout。

        //1、创建窗口对象
        frame = new JFrame("高拍仪");

        Container c = frame.getContentPane();//获取窗体主容器
        c.setLayout(null);


        //左侧高拍仪画面区域
        JPanel leftPanel = new JPanel();
        leftPanel.setBorder(BorderFactory.createTitledBorder("摄像头"));
        leftPanel.setBounds(10,0,660,520);

        JLabel textArea = new JLabel();
        Font font = new Font("Serif", Font.PLAIN, 24);
        textArea.setFont(font);
        textArea.setText("正在加载摄像头...");
        textArea.setForeground(Color.black);
        textArea.setVisible(true);
        leftPanel.add(textArea);

//        videoCamera.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));

        new CameraOprationByOpenCV().invokeCamera(videoCamera,textArea);


        frame.add(videoCamera);
        frame.add(leftPanel);


        //右侧操作区域
        JPanel rightPanel = new JPanel();
        rightPanel.setBounds(680,0,300,520);
        rightPanel.setBorder(BorderFactory.createTitledBorder("操作区"));


        //右侧上方单选按钮区域
        JPanel radioPanel1 = new JPanel(new GridLayout(1,4,10,10));
        radioPanel1.setBounds(700,20,260,50);

        ButtonGroup cbg = new ButtonGroup();
        JRadioButton nocut = new JRadioButton("",cutType == 0?true:false);
        nocut.setIcon(ImageUtil.getImage("img/cut_no.png"));
        nocut.setSelectedIcon(ImageUtil.getImage("img/cut_no_check.png"));

        JRadioButton autocut = new JRadioButton("",cutType == 1?true:false);
        autocut.setIcon(ImageUtil.getImage("img/cut_auto.png"));
        autocut.setSelectedIcon(ImageUtil.getImage("img/cut_auto_check.png"));

        JRadioButton mycut = new JRadioButton("",cutType == 2?true:false);
        mycut.setIcon(ImageUtil.getImage("img/cut_customize.png"));
        mycut.setSelectedIcon(ImageUtil.getImage("img/cut_customize_check.png"));

        //占位按钮  不然会显3列,写一个错误的图片地址
        JRadioButton blank = new JRadioButton("",new ImageIcon(""),false);

        nocut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "不切边");
                cutType = 0;
                writeConfig();
            }
        });
        autocut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "自动切边");
                cutType = 1;
                writeConfig();
            }
        });
        mycut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "自定义切边");
                cutType = 2;
                writeConfig();
            }
        });

        blank.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(count == 0){
                    cameraList();  //读取设备摄像头数量
                }
                // 创建对话框
                setDialog = new JDialog();
                setDialog.setTitle("配置");
                setDialog.setModal(true); // 设置为模式对话框
                setDialog.setBounds(760,390,400, 300);

                // 创建FlowLayout布局管理器并设置间距
                FlowLayout layout = new FlowLayout();
                layout.setHgap(10);
                layout.setVgap(10);
                setDialog.setLayout(layout);

                //摄像头列表
                JLabel cameraLabel = new JLabel("摄像头:");
                JComboBox<String> cameraBox = new JComboBox<String>();//创建一个下拉列表框
                cameraBox.setBounds(110,11,80,21);        //设置坐标
                if(count > 0){
                    for (int i = 0; i < count; i++) {
                        cameraBox.addItem("Camera "+(i+1));
                    }
                    if(cameraIndex < count){
                        cameraBox.setSelectedIndex(cameraIndex);
                    }
                }
                cameraBox.setEditable(true);            //将下拉列表添加到容器中
                cameraBox.addItemListener(new ItemListener() {
                    @Override
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.SELECTED) {
                            cameraIndex = cameraBox.getSelectedIndex();
                            writeConfig();
                            setDialog.dispose();
                            JOptionPane.showMessageDialog(frame, "修改成功，请重启！");

                        }
                    }
                });

                //分辨率
                JLabel whLabel = new JLabel("分辨率:");
                JComboBox<String> whBox = new JComboBox<String>();//创建一个下拉列表框
                whBox.setBounds(110,11,80,21);        //设置坐标
//                    whBox.addItem("4224x3136");
//                    whBox.addItem("4096x3072");
//                    whBox.addItem("1920x1088");
//                    whBox.addItem("4096x3072");
//                    whBox.addItem("1920x1088");
                whBox.addItem("1920x1080");
                whBox.addItem("1280x720");
                whBox.addItem("720x480");
                whBox.addItem("640x480");
//                    whBox.addItem("352x288");
//                    whBox.addItem("320x240");
//                    whBox.addItem("176x144");
                whBox.setEditable(true);            //将下拉列表添加到容器中
                whBox.setSelectedItem(ratio);
                whBox.addItemListener(new ItemListener() {
                    @Override
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == ItemEvent.SELECTED) {
                            ratio = (String) whBox.getSelectedItem();
                            writeConfig();
                            setDialog.dispose();
                            JOptionPane.showMessageDialog(frame, "修改成功，请重启！");

                        }
                    }
                });

                //照片张数限制
                JLabel maxLabel = new JLabel("最大拍照张数:");
                JTextField jt = new JTextField();
                if(maxPhoto == 0 ){
                    jt.setText("不限制");
                }else {
                    jt.setText(maxPhoto+"");
                }
                jt.setColumns(14);//设置文本框长度
//                    jt.setFont(new Font("宋体", Font.PLAIN, 12));//设置字体
                JButton jb = new JButton("保存");
                jb.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        if(jt.getText().equals("不限制")){
                            maxPhoto = 0;
                            writeConfig();
                            JOptionPane.showMessageDialog(frame, "保存成功");
                        }else if(isNumeric(jt.getText()) && Integer.parseInt(jt.getText()) >= 0){
                            maxPhoto = Integer.parseInt(jt.getText());
                            writeConfig();
                            JOptionPane.showMessageDialog(frame, "保存成功");
                        }else {
                            JOptionPane.showMessageDialog(frame, "请输入大于0的整数");
                        }
                    }
                });

                setDialog.add(cameraLabel);
                setDialog.add(cameraBox);

                setDialog.add(whLabel);
                setDialog.add(whBox);

                setDialog.add(maxLabel);
                setDialog.add(jt);
                setDialog.add(jb);

                setDialog.setVisible(true);
            }
        });

        cbg.add(nocut);
        cbg.add(autocut);
        cbg.add(mycut);

        radioPanel1.add(nocut);
        radioPanel1.add(autocut);
        radioPanel1.add(mycut);
        radioPanel1.add(blank);

        //添加第一排单选按钮
        frame.add(radioPanel1);

        JPanel radioPanel2 = new JPanel(new GridLayout(1,4,10,10));
        radioPanel2.setBounds(700,80,260,50);

        ButtonGroup cbg1 = new ButtonGroup(); //一组Checkbox
        JRadioButton norotate = new JRadioButton("",roteAngle == 0 ? true:false);
        norotate.setIcon(ImageUtil.getImage("img/rotate_no.png"));
        norotate.setSelectedIcon(ImageUtil.getImage("img/rotate_no_check.png"));

        JRadioButton rotate90 = new JRadioButton("",roteAngle == 90 ? true:false);
        rotate90.setIcon(ImageUtil.getImage("img/rotate_90.png"));
        rotate90.setSelectedIcon(ImageUtil.getImage("img/rotate_90_check.png"));

        JRadioButton rotate180 = new JRadioButton("",roteAngle == 180 ? true:false);
        rotate180.setIcon(ImageUtil.getImage("img/rotate_180.png"));
        rotate180.setSelectedIcon(ImageUtil.getImage("img/rotate_180_check.png"));

        JRadioButton rotate270= new JRadioButton("",roteAngle == 270 ? true:false);
        rotate270.setIcon(ImageUtil.getImage("img/rotate_270.png"));
        rotate270.setSelectedIcon(ImageUtil.getImage("img/rotate_270_check.png"));


        norotate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "不旋转");
                roteAngle = 0;
                writeConfig();
            }
        });
        rotate90.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转90°");
                roteAngle = 90;
                writeConfig();
            }
        });
        rotate180.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转180°");
                roteAngle = 180;
                writeConfig();
            }
        });
        rotate270.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转270°");
                roteAngle = 270;
                writeConfig();
            }
        });

        cbg1.add(norotate);
        cbg1.add(rotate90);
        cbg1.add(rotate180);
        cbg1.add(rotate270);

        radioPanel2.add(norotate);
        radioPanel2.add(rotate90);
        radioPanel2.add(rotate180);
        radioPanel2.add(rotate270);

        //添加第二排单选按钮
        frame.add(radioPanel2);


        //添加拍照图片显示区域
        //照片列表
        ArrayList<BufferedImage> photoList = new ArrayList<>();
        JPanel picPanel = new JPanel(new GridLayout(0,2,10,10));
        picPanel.setSize(0,0);

        //创建 JScrollPane 滚动面板，并将文本域放到滚动面板中
        JScrollPane sp = new JScrollPane(picPanel);
        sp.setBorder(null);
        sp.setBounds(690,150,280,300);
        sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        sp.setVisible(true);
        c.add(sp);


        //拍照区域按钮
        JPanel subPanel = new JPanel(new GridLayout(1,2,10,10));
        subPanel.setBounds(700,460,260,50);

        JButton takPicBtn = new JButton();
        //去除边框和背景色
        takPicBtn.setBorderPainted(false);
        takPicBtn.setContentAreaFilled(false);
        takPicBtn.setFocusPainted(false);
        takPicBtn.setOpaque(false);
        takPicBtn.setIcon(ImageUtil.getImage("img/shoot.png"));
        takPicBtn.setPressedIcon(ImageUtil.getImage("img/shoot_hover.png"));
        takPicBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "拍照");
                if(photoList.size() < maxPhoto || maxPhoto == 0){
                    log.info(">>>>>>take photo performed");
                    try {
                        if (!outMat.empty()) {
//                          Image previewImg = ImageUtils.scale2(myFace, 165, 200, true);// 等比例缩放
                            BufferedImage previewImg = null;   //预览图
                            BufferedImage souceImg = null;  //没缩放的大图

                            if (cutType == 1 || cutType == 2) {
                                if(videoCamera.getRect() != null){
                                    souceImg = OpencvHelper.ImageCutByRect(outMat,videoCamera.getRect());
                                }
                            }else {
                                souceImg = OpencvHelper.matToBufferedImage(outMat);
                            }

                            previewImg = OpencvHelper.ImageZoom(OpencvHelper.imageToMat(souceImg),100,120);

//                          TakePhotoProcess takePhoto = new TakePhotoProcess(photoPath.toString(), myFhoto);
//                          takePhoto.start();// 照片写盘

                            photoList.add(previewImg);
                            JLabel jLabel = new MyImageLabel(new ImageIcon(previewImg));
                            BufferedImage finalSouceImg = souceImg;
                            BufferedImage finalPreviewImg = previewImg;
                            jLabel.addMouseListener(new MouseAdapter() {
                                @Override
                                public void mouseClicked(MouseEvent e) {
                                    super.mouseClicked(e);
                                    if(e.getX() > 76 && e.getY() < 24){//删除图标的位置
                                        photoList.remove(finalPreviewImg);
                                        picPanel.remove(jLabel);
                                        // 刷新JScrollPane
                                        sp.revalidate();
                                        sp.repaint();
                                        JScrollBar scrollBar = sp.getVerticalScrollBar();
                                        // 将滚动条位置设置为最大值，以滚动到最底部
                                        scrollBar.setValue(scrollBar.getMaximum());
                                    }else if(e.getY() < finalPreviewImg.getHeight()){  //图片位置
                                        // 创建对话框
                                        preDialog = new JDialog();
                                        preDialog.setTitle("预览图");
                                        preDialog.setModal(true); // 设置为模式对话框
//                                                preDialog.setBounds(560,240,800, 600);
                                        preDialog.setMinimumSize(new Dimension(800, 600));
                                        preDialog.setSize(finalSouceImg.getWidth(), finalSouceImg.getHeight());

                                        preDialog.setLayout(new BorderLayout());

//                                                JLabel imgJlable = new JLabel(new ImageIcon(finalSouceImg));
                                        ImagePanle mImgeView = new ImagePanle();
                                        mImgeView.setPreferredSize(new Dimension(500, 500));
                                        mImgeView.setMinimumSize(new Dimension(500, 500));
                                        mImgeView.updateImage(finalSouceImg);

                                        preDialog.add(mImgeView, BorderLayout.CENTER);

                                        preDialog.setVisible(true);
                                    }

                                }
                            });

                            picPanel.add(jLabel);
                            // 刷新JScrollPane
                            sp.revalidate();
                            sp.repaint();
                            JScrollBar scrollBar = sp.getVerticalScrollBar();
                            // 将滚动条位置设置为最大值，以滚动到最底部
                            scrollBar.setValue(scrollBar.getMaximum());
                        }else {
                            log.error("拍照失败->outMat : " + outMat);
                        }
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(frame, "请在图像区域内画框");
                        log.error(">>>>>>take photo error: " + ex.getMessage());
                    }
                }else {
                    JOptionPane.showMessageDialog(frame, "最大拍照张数为："+maxPhoto);
                }

            }
        });
        subPanel.add(takPicBtn);

        JButton subPicBtn = new JButton();
        subPicBtn.setBorderPainted(false);
        subPicBtn.setContentAreaFilled(false);
        subPicBtn.setFocusPainted(false);
        subPicBtn.setOpaque(false);
        subPicBtn.setIcon(ImageUtil.getImage("img/upload.png"));
        subPicBtn.setPressedIcon(ImageUtil.getImage("img/upload_hover.png"));
        subPicBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "上传");
                if(photoList.size() > 0){
                    photoBeanArrayList.clear();
                    for (int i = 0; i < photoList.size(); i++) {
                        photoBeanArrayList.add(new PhotoBean(System.currentTimeMillis()+".png",OpencvHelper.imageToBase64(photoList.get(i))));
                    }

                    frame.dispose();
                }else {
                    JOptionPane.showMessageDialog(frame, "未拍摄照片");
                }

            }
        });
        subPanel.add(subPicBtn);

        //添加拍照区按钮
        frame.add(subPanel);

        //添加右侧区域
        frame.add(rightPanel);

        //文字说明区
        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.setBorder(BorderFactory.createTitledBorder("说明"));
        bottomPanel.setBounds(10,530,970,40);

        JLabel textArea1 = new JLabel();
        Font font1 = new Font("Serif", Font.PLAIN, 10);
        textArea1.setFont(font1);
        textArea1.setForeground(Color.black);
        textArea1.setVisible(true);


        textArea1.setText("  软件版本 V1.0.1    摄像头：Camera "+(cameraIndex+1) +"      分辨率："+ratio+"      最大拍照张数："+ (maxPhoto==0?"不限制":maxPhoto));
        bottomPanel.add(textArea1,BorderLayout.CENTER);

        //添加文字说明区
        frame.add(bottomPanel);

        //合适的窗口大小
        frame.pack();
        //禁止改变窗口大小
        frame.setResizable(false);
        //设置窗口关闭退出软件  默认 JFrame.EXIT_ON_CLOSE会关闭所有窗口   DISPOSE_ON_CLOSE仅关闭当前窗口
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        //设置窗口的位置和大小
        frame.setBounds(460,240,1000,610);
        //设置窗口对象可见
        frame.setVisible(true);

        frame.addWindowListener(new WindowListener() {
            @Override
            public void windowOpened(WindowEvent e) {
            }

            @Override
            public void windowClosing(WindowEvent e) {

            }

            @Override
            public void windowClosed(WindowEvent e) {
                timer = 0;  //手动关闭窗口时重置timer 停止while循环 并返回超时
                //窗口关闭释放摄像头资源
                if(capture != null){
                    capture.release();
                }

                if(setDialog != null && setDialog.isShowing()){
                    setDialog.dispose();
                }

                if(preDialog != null && preDialog.isShowing()){
                    preDialog.dispose();
                }
            }

            @Override
            public void windowIconified(WindowEvent e) {
            }

            @Override
            public void windowDeiconified(WindowEvent e) {
            }

            @Override
            public void windowActivated(WindowEvent e) {
            }

            @Override
            public void windowDeactivated(WindowEvent e) {
            }
        });

        while (true) {

            try {
                timer--;
                frame.setTitle("高拍仪（操作倒计时: "+timer+" 秒）");
                Thread.sleep(1000);
            } catch (InterruptedException e) {

            }


            if (!ObjectUtils.isEmpty(photoBeanArrayList) && photoBeanArrayList.size() > 0) {
                Rest<ArrayList<PhotoBean>> rest = Rest.ok();
                rest.setData(photoBeanArrayList);
                return rest;
            }

            if(timer <= 0){
                Rest<ArrayList<PhotoBean>> rest = Rest.fail();
                rest.setData(null);
                rest.setMsg("操作超时");

                if(frame.isShowing()){
                    frame.dispose();
                }
                return rest;
            }
        }

    }



    public void invokeCamera(VideoPanel videoPanel,JLabel textArea) {
        new Thread(() -> {
            try {
                capture = new VideoCapture(cameraIndex);
                log.info("原始分辨率: " + capture.get(CAP_PROP_FRAME_WIDTH) + "x" + capture.get(CAP_PROP_FRAME_HEIGHT));
                // 设置新的分辨率
                String[] wh = ratio.split("x");
                int newWidth = 640;
                int newHeight = 480;
                if(wh.length == 2){
                    newWidth = Integer.parseInt(wh[0]); // 新的宽度
                    newHeight = Integer.parseInt(wh[1]); // 新的高度

                    capture.set(CAP_PROP_FRAME_WIDTH, newWidth);
                    capture.set(CAP_PROP_FRAME_HEIGHT, newHeight);

                    //计算图像的宽高  按宽度等比例缩放
                    preHeight = 640 * newHeight / newWidth;
                    log.info("当前设置分辨率: " + newWidth + "x" + newHeight);
                }

                videoCamera.setBounds(20,20+(480 - preHeight)/2,640,preHeight);    //让画面左右撑满  上下居中
                log.info("视图位置："+videoCamera.getX()+","+videoCamera.getY()+","+videoCamera.getWidth()+","+videoCamera.getHeight());

                if (capture.isOpened()) {
                    textArea.setVisible(false);
                    videoPanel.setVisible(true);
                    sourceMat = new Mat();
                    outMat = new Mat();
                    while (true) {
                        capture.read(sourceMat);

                        outMat = OpencvHelper.ImageZoomMat(OpencvHelper.ImageRotateMat(sourceMat,roteAngle),640,preHeight);
                        Rectangle rectangle = null;

                        if(cutType == 1){  //自动切边时需要实时画框
                            rectangle = OpencvHelper.cutAutoGetAngele(outMat);
                        }

                        videoPanel.setImageWithMat(outMat,rectangle,cutType);

                        Thread.sleep(10);
                    }
                }else {
                    log.error("invoke camera error : 摄像头打开失败");
                    textArea.setText("摄像头打开失败，请检查插线连接");
                    textArea.setVisible(true);
                    videoPanel.setVisible(false);
                }
            } catch (Exception e) {
                log.error("invoke camera error: " + e.toString());
                textArea.setText("摄像头掉线，请检查插线连接");
                textArea.setVisible(true);
                videoPanel.setVisible(false);
            }
        }).start();
    }

    /**
     * 读取摄像头列表
     */
    private static VideoCapture[] cameraList(){
        // 创建VideoCapture对象列表，用于存储摄像头设备
        VideoCapture[] captures = new VideoCapture[5];

        // 遍历系统中的摄像头设备
        for (int i = 0; i < captures.length; i++) {
            try {
                // 尝试打开摄像头设备
                captures[i] = new VideoCapture(i);
                if (captures[i].isOpened()) {
                    count++;
                } else {
                    captures[i].release(); // 如果无法打开，释放资源
                }
            } catch (Exception e) {

                e.printStackTrace();
            }
        }

//        // 输出摄像头列表信息
        log.info("找到 " + count + " 个摄像头设备：");


        return captures;
    }


    /**
     * 判断是否为数字
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        if (str == null || str.length() == 0) {
            return false;
        }
        for (int i = 0; i < str.length(); i++) {
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }
}

