package com.mortals.xhx.opencv.UI;


import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.opencv.bean.ConfigBean;
import com.mortals.xhx.opencv.bean.PhotoBean;
import com.mortals.xhx.opencv.utils.ImageUtil;
import com.mortals.xhx.opencv.utils.OpencvHelper;
import lombok.extern.slf4j.Slf4j;
import org.bytedeco.opencv.opencv_core.Mat;
import org.bytedeco.opencv.opencv_videoio.VideoCapture;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.springframework.util.ObjectUtils;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Scanner;

import static org.bytedeco.opencv.global.opencv_videoio.CAP_PROP_FRAME_HEIGHT;
import static org.bytedeco.opencv.global.opencv_videoio.CAP_PROP_FRAME_WIDTH;


/**
 * @author ZYW
 * @date 2024-01-08 13:55
 */
@Slf4j
public class CaptureUI {


    static String configPath = "F:\\\\config.prop";  //默认测试本机路径

    private static CaptureUI instance = null;
    static JFrame frame;


    static Dialog setDialog;   //配置框
    static Dialog preDialog;   //预览照片dialog

    protected static CapturePanel videoCamera = new CapturePanel();

    public static ArrayList<PhotoBean> photoBeanArrayList = new ArrayList<>();

    static Gson gson = new Gson();
    public static int cutType = 0;  //0不切边  1自动切边  2 自定义切边
    public static int roteAngle = 0;   //旋转角度
    public static int cameraIndex = 0;   //选中的摄像头
    public static String cameraName = "";   //选中的摄像头名称
    public static int maxPhoto = 0;     //照片最大拍照张数  0为不限制
    public static String ratio = "640x480";     //摄像头默认分辨率

    static int timer = 300;   //用于两分钟无操作计时

    static Mat outMat; //缩放后的图像

    static String photoPath = "/tmp/photo/";  //本地拍照图片路劲


    static WebSocketClient webSocketClient = null;

    public static void main(String[] args) {
        //初始化窗口
        initWindow(300,configPath,"ws://192.168.0.158:1818/");
    }



    private CaptureUI() {
    }

    public static CaptureUI getInstance() {
        if (null == instance) {
            synchronized (CaptureUI.class) {
                if (null == instance) {
                    instance = new CaptureUI();
                }
            }
        }

        return instance;
    }

    /**
     * 读取本地文件配置
     */
    private static void redConfig(){
        File file = new File(configPath);
        try {
            file.createNewFile();
        } catch (IOException e) {
            log.error( "创建文件失败" ); ;
        }

        Scanner sc = null ;
        try
        {
            sc = new Scanner( file ) ;

            String jsonString = sc.next();
            ConfigBean jo = gson.fromJson(jsonString,ConfigBean.class);


            if(jo != null){
                cutType = jo.getCutType();
                roteAngle = jo.getRoteAngle();
                cameraIndex = jo.getCameraIndex();
                maxPhoto = jo.getMaxPhoto();
                if(jo.getRatio() != null){
                    ratio = jo.getRatio();
                }
            }else {
                log.info( "配置文件为空" ); ;
            }


        }
        catch( Exception e )
        {
            log.error( "读取配置文件失败"+e.toString() ); ;
        }
        finally
        {
            try
            {
//                log.error( "读写关闭" ) ;
                sc.close();
            }
            catch ( Exception e )
            {

            }
        }
    }

    /**
     * 修改本地文件配置
     */
    private static void writeConfig(){

        ConfigBean cb = new ConfigBean(cutType,roteAngle,cameraIndex,maxPhoto,ratio);

        File file = new File(configPath);
        try {
            file.createNewFile();
        } catch (IOException e) {
        }

        FileWriter fileWriter = null ;
        BufferedWriter bufferedWriter = null ;  //FileWriter和BufferedWriter的初始化需要监听，此处只定义不初始化

        try
        {
            fileWriter = new FileWriter( file ) ;
            bufferedWriter = new BufferedWriter( fileWriter ) ;

            bufferedWriter.write( gson.toJson(cb) ) ;
        }
        catch( Exception e )
        {
            log.error( "Error." ); ;
        }
        finally
        {
            try
            {

                if(bufferedWriter != null){
                    bufferedWriter.close(); //随手关闭文件,此处注意关闭顺序不能错误
                }
                if(fileWriter != null){
                    fileWriter.close();
                }
            }
            catch( IOException ioe )
            {

            }

        }

    }

    /**
     * 初始化窗口
     */
    public static Rest<ArrayList<PhotoBean>> initWindow(int runtime,String path,String captureUrl){
        configPath = path;
        photoBeanArrayList.clear();
        File folder = new File(photoPath);
        if(!folder.exists()){
            folder.mkdir();
        }else {
            // 获取文件夹下所有文件
            File[] files = folder.listFiles();

            if (files != null) {
                for (File file : files) {
                    if (file.isFile()) {
                        file.delete(); // 删除文件
                    } else if (file.isDirectory()) {
                        deleteFilesInFolder(file); // 递归删除子文件夹下的文件
                    }
                }
            }
        }

        redConfig();
        timer = runtime;
        //AWT中接口LayoutManager有五个实现类：GridLayout（网格布局）、FlowLayout（流式布局）、CardLayout（卡片布局）、
        // GridBagLayout（网格包布局）和BorderLayout（边框布局）。为了简化开发，Swing引入了一个新的布局管理器BoxLayout。

        //1、创建窗口对象
        frame = new JFrame("高拍仪");

        Container c = frame.getContentPane();//获取窗体主容器
        c.setLayout(null);


        //左侧高拍仪画面区域
        JPanel leftPanel = new JPanel();
        leftPanel.setBorder(BorderFactory.createTitledBorder("摄像头"));
        leftPanel.setBounds(10,0,820,630);

        JLabel textArea = new JLabel();
        Font font = new Font("Serif", Font.PLAIN, 24);
        textArea.setFont(font);
        textArea.setText("正在加载摄像头...");
        textArea.setForeground(Color.black);
        textArea.setVisible(true);
        leftPanel.add(textArea);

//        videoCamera.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
        videoCamera.setBounds(20,20,800,600);    //让画面左右撑满  上下居中
        initWebsocket(videoCamera,captureUrl,textArea);


        frame.add(videoCamera);
        frame.add(leftPanel);


        //右侧操作区域
        JPanel rightPanel = new JPanel();
        rightPanel.setBounds(840,0,300,630);
        rightPanel.setBorder(BorderFactory.createTitledBorder("操作区"));


        //右侧上方单选按钮区域
        JPanel radioPanel1 = new JPanel(new GridLayout(1,4,10,10));
        radioPanel1.setBounds(860,20,260,50);

        ButtonGroup cbg = new ButtonGroup();
        JRadioButton nocut = new JRadioButton("",cutType == 0?true:false);
        nocut.setIcon(ImageUtil.getImage("img/cut_no.png"));
        nocut.setSelectedIcon(ImageUtil.getImage("img/cut_no_check.png"));

        JRadioButton autocut = new JRadioButton("",cutType == 1?true:false);
        autocut.setIcon(ImageUtil.getImage("img/cut_auto.png"));
        autocut.setSelectedIcon(ImageUtil.getImage("img/cut_auto_check.png"));

        JRadioButton mycut = new JRadioButton("",cutType == 2?true:false);
        mycut.setIcon(ImageUtil.getImage("img/cut_sfz.png"));
        mycut.setSelectedIcon(ImageUtil.getImage("img/cut_sfz_check.png"));

        //占位按钮  不然会显3列,写一个错误的图片地址
        JRadioButton blank = new JRadioButton("",new ImageIcon(""),false);

        nocut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "不切边");
                sendMess("bSetMode(0)");
                cutType = 0;
                writeConfig();
            }
        });
        autocut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "自动切边");
                sendMess("bSetMode(3)");
                cutType = 1;
                writeConfig();
            }
        });
        mycut.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "自定义切边");
                sendMess("bSetMode(4)");
                cutType = 2;
                writeConfig();
            }
        });


        cbg.add(nocut);
        cbg.add(autocut);
        cbg.add(mycut);

        radioPanel1.add(nocut);
        radioPanel1.add(autocut);
        radioPanel1.add(mycut);
        radioPanel1.add(blank);

        //添加第一排单选按钮
        frame.add(radioPanel1);

        JPanel radioPanel2 = new JPanel(new GridLayout(1,4,10,10));
        radioPanel2.setBounds(860,80,260,50);

        ButtonGroup cbg1 = new ButtonGroup(); //一组Checkbox
        JRadioButton norotate = new JRadioButton("",roteAngle == 0 ? true:false);
        norotate.setIcon(ImageUtil.getImage("img/rotate_no.png"));
        norotate.setSelectedIcon(ImageUtil.getImage("img/rotate_no_check.png"));

        JRadioButton rotate90 = new JRadioButton("",roteAngle == 90 ? true:false);
        rotate90.setIcon(ImageUtil.getImage("img/rotate_90.png"));
        rotate90.setSelectedIcon(ImageUtil.getImage("img/rotate_90_check.png"));

        JRadioButton rotate180 = new JRadioButton("",roteAngle == 180 ? true:false);
        rotate180.setIcon(ImageUtil.getImage("img/rotate_180.png"));
        rotate180.setSelectedIcon(ImageUtil.getImage("img/rotate_180_check.png"));

        JRadioButton rotate270= new JRadioButton("",roteAngle == 270 ? true:false);
        rotate270.setIcon(ImageUtil.getImage("img/rotate_270.png"));
        rotate270.setSelectedIcon(ImageUtil.getImage("img/rotate_270_check.png"));


        norotate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "不旋转");
                roteAngle = 0;
                writeConfig();
            }
        });
        rotate90.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转90°");
                roteAngle = 90;
                writeConfig();
            }
        });
        rotate180.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转180°");
                roteAngle = 180;
                writeConfig();
            }
        });
        rotate270.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "旋转270°");
                roteAngle = 270;
                writeConfig();
            }
        });

        cbg1.add(norotate);
        cbg1.add(rotate90);
        cbg1.add(rotate180);
        cbg1.add(rotate270);

        radioPanel2.add(norotate);
        radioPanel2.add(rotate90);
        radioPanel2.add(rotate180);
        radioPanel2.add(rotate270);

        //添加第二排单选按钮
        frame.add(radioPanel2);


        //添加拍照图片显示区域
        //照片列表
        ArrayList<File> photoList = new ArrayList<>();
        JPanel picPanel = new JPanel(new GridLayout(0,2,10,10));
        picPanel.setSize(0,0);

        //创建 JScrollPane 滚动面板，并将文本域放到滚动面板中
        JScrollPane sp = new JScrollPane(picPanel);
        sp.setBorder(null);
        sp.setBounds(850,150,280,400);
        sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        sp.setVisible(true);
        c.add(sp);


        //拍照区域按钮
        JPanel subPanel = new JPanel(new GridLayout(1,2,10,10));
        subPanel.setBounds(860,580,260,50);

        JButton takPicBtn = new JButton();
        //去除边框和背景色
        takPicBtn.setBorderPainted(false);
        takPicBtn.setContentAreaFilled(false);
        takPicBtn.setFocusPainted(false);
        takPicBtn.setOpaque(false);
        takPicBtn.setIcon(ImageUtil.getImage("img/shoot.png"));
        takPicBtn.setPressedIcon(ImageUtil.getImage("img/shoot_hover.png"));
        takPicBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "拍照");
                if(photoList.size() < maxPhoto || maxPhoto == 0){
                    long picName = System.currentTimeMillis();
                    sendMess("bSaveJPG("+photoPath+","+picName+")");

//                    try {
//                        Thread.sleep(1000);
//                    } catch (InterruptedException ex) {
//                        throw new RuntimeException(ex);
//                    }

                    //没缩放的大图
                    File souceImg = new File(photoPath+picName+".jpg");
                    while (!souceImg.exists()){
                        try {
                            Thread.sleep(10);
                        } catch (InterruptedException ex) {
                            throw new RuntimeException(ex);
                        }
                    }

                    photoList.add(souceImg);
                    JLabel jLabel = new MyImageLabel(ImageUtil.zoomImageByFile(souceImg,100,120));
                    jLabel.setSize(100,120);
                    jLabel.addMouseListener(new MouseAdapter() {
                        @Override
                        public void mouseClicked(MouseEvent e) {
                            super.mouseClicked(e);
                            if(e.getX() > 76 && e.getY() < 24){//删除图标的位置
                                photoList.remove(souceImg);
                                picPanel.remove(jLabel);
                                if(souceImg.exists()){
                                    souceImg.delete();
                                }
                                // 刷新JScrollPane
                                sp.revalidate();
                                sp.repaint();
                                JScrollBar scrollBar = sp.getVerticalScrollBar();
                                // 将滚动条位置设置为最大值，以滚动到最底部
                                scrollBar.setValue(scrollBar.getMaximum());
                            }else if(e.getY() < 120){  //图片位置
                                // 创建对话框
                                preDialog = new JDialog();
                                preDialog.setTitle("预览图");
                                preDialog.setModal(true); // 设置为模式对话框
//                                                preDialog.setBounds(560,240,800, 600);
                                preDialog.setMinimumSize(new Dimension(800, 600));
                                preDialog.setSize(800, 600);

                                preDialog.setLayout(new BorderLayout());

                                JLabel mImgeView = new JLabel(ImageUtil.getImageByFile(souceImg));
//                                ImagePanle mImgeView = new ImagePanle();
//                                mImgeView.setPreferredSize(new Dimension(500, 500));
//                                mImgeView.setMinimumSize(new Dimension(500, 500));
//                                mImgeView.updateImage(finalSouceImg);

                                preDialog.add(mImgeView, BorderLayout.CENTER);

                                preDialog.setVisible(true);
                            }

                        }
                    });

                    picPanel.add(jLabel);
                    // 刷新JScrollPane
                    sp.revalidate();
                    sp.repaint();
                    JScrollBar scrollBar = sp.getVerticalScrollBar();
                    // 将滚动条位置设置为最大值，以滚动到最底部
                    scrollBar.setValue(scrollBar.getMaximum());
                }else {
                    JOptionPane.showMessageDialog(frame, "最大拍照张数为："+maxPhoto);
                }

            }
        });
        subPanel.add(takPicBtn);

        JButton subPicBtn = new JButton();
        subPicBtn.setBorderPainted(false);
        subPicBtn.setContentAreaFilled(false);
        subPicBtn.setFocusPainted(false);
        subPicBtn.setOpaque(false);
        subPicBtn.setIcon(ImageUtil.getImage("img/upload.png"));
        subPicBtn.setPressedIcon(ImageUtil.getImage("img/upload_hover.png"));
        subPicBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
//                JOptionPane.showMessageDialog(frame, "上传");
                if(photoList.size() > 0){
                    photoBeanArrayList.clear();

                    File[] files = folder.listFiles();

                    if (files != null) {
                        for (File file : files) {
                            if (file.isFile() && file.getName().endsWith(".jpg")) {
                                photoBeanArrayList.add(new PhotoBean(file.getName(),imageToBase64(file)));
                            }
                        }
                    }


                    frame.dispose();
                }else {
                    JOptionPane.showMessageDialog(frame, "未拍摄照片");
                }

            }
        });
        subPanel.add(subPicBtn);

        //添加拍照区按钮
        frame.add(subPanel);

        //添加右侧区域
        frame.add(rightPanel);


        //合适的窗口大小
        frame.pack();
        //禁止改变窗口大小
        frame.setResizable(false);
        //设置窗口关闭退出软件  默认 JFrame.EXIT_ON_CLOSE会关闭所有窗口   DISPOSE_ON_CLOSE仅关闭当前窗口
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        //设置窗口的位置和大小
        frame.setBounds(380,200,1160,670);
        //设置窗口对象可见
        frame.setVisible(true);

        frame.addWindowListener(new WindowListener() {
            @Override
            public void windowOpened(WindowEvent e) {
            }

            @Override
            public void windowClosing(WindowEvent e) {

            }

            @Override
            public void windowClosed(WindowEvent e) {
                timer = 0;  //手动关闭窗口时重置timer 停止while循环 并返回超时

                if(setDialog != null && setDialog.isShowing()){
                    setDialog.dispose();
                }

                if(preDialog != null && preDialog.isShowing()){
                    preDialog.dispose();
                }
            }

            @Override
            public void windowIconified(WindowEvent e) {
            }

            @Override
            public void windowDeiconified(WindowEvent e) {
            }

            @Override
            public void windowActivated(WindowEvent e) {
            }

            @Override
            public void windowDeactivated(WindowEvent e) {
            }
        });

        while (true) {

            try {
                timer--;
                frame.setTitle("高拍仪（操作倒计时: "+timer+" 秒）");
                Thread.sleep(1000);
            } catch (InterruptedException e) {

            }


            if (!ObjectUtils.isEmpty(photoBeanArrayList) && photoBeanArrayList.size() > 0) {
                Rest<ArrayList<PhotoBean>> rest = Rest.ok();
                rest.setData(photoBeanArrayList);
                destroyWebsocket();
                return rest;
            }

            if(timer <= 0){
                Rest<ArrayList<PhotoBean>> rest = Rest.fail();
                rest.setData(null);
                rest.setMsg("操作超时");

                if(frame.isShowing()){
                    frame.dispose();
                }
                destroyWebsocket();
                return rest;
            }
        }

    }


    private static void initWebsocket(CapturePanel capturePanel, String captureUrl,JLabel textArea){
        URI uri;
        try {
            uri = new URI(captureUrl);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        webSocketClient = new WebSocketClient(uri) {
            @Override
            public void onOpen(ServerHandshake handshakedata) {
                log.info("新连接已打开");
                send("bStopPlay");
                send("bStartPlay");
                if(cutType == 2){
                    send("bSetMode(3)");
                }else if(cutType == 3){
                    send("bSetMode(4)");
                }else {
                    send("bSetMode(0)");
                }
            }

            @Override
            public void onMessage(String message) {
                if(!message.startsWith("/9j/")){
                    log.info("接收到消息: {}",message);
                }
                int strBegin = message.indexOf("Begin");
                if(0 == strBegin)
                {
                    log.info("接收到消息->strBegin: {}",strBegin);
                    //alert(received_msg);
                }
                else
                {
                    textArea.setVisible(false);
                    capturePanel.setVisible(true);
                    String img = "data:image/jpeg;base64,"+message;
//                    log.info("图片: {}",img);
                    try {
                        capturePanel.decodeToImage(message);
                    } catch (IOException e) {
                        textArea.setVisible(true);
                        capturePanel.setVisible(false);
                        textArea.setText("base64解码失败");
                        throw new RuntimeException(e);

                    }
                }

            }

            @Override
            public void onClose(int code, String reason, boolean remote) {
                log.info("连接已关闭,code:{} reason:{} {}",code,reason,remote);
                textArea.setVisible(true);
                capturePanel.setVisible(false);
                textArea.setText("连接已关闭,code:"+code+" reason:"+reason);
            }

            @Override
            public void onError(Exception ex) {
                ex.printStackTrace();
                log.info("链接出错了"+ex.toString());
                textArea.setVisible(true);
                capturePanel.setVisible(false);
                textArea.setText("链接出错了"+ex.getMessage());
            }
        };
        webSocketClient.connect(); // 连接到WebSocket服务器
    }

    /**
     * 关闭socket链接  关闭摄像头
     */
    static void sendMess(String mess){
        if(webSocketClient != null){
            webSocketClient.send(mess);
        }
    }

    /**
     * 关闭socket链接  关闭摄像头
     */
    static void destroyWebsocket(){
        if(webSocketClient != null){
            webSocketClient.send("bStopPlay");
        }
        webSocketClient.close();
    }

    /**
     * 本地图片转BufferedImage
     * @param imagePath
     * @return
     */
    public static BufferedImage convertToBufferedImage(String imagePath) {
        try {
            log.info("imagePath:{}",imagePath);
            File file = new File(imagePath);
            return ImageIO.read(file);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 删除文件
     * @param folder
     */
    private static void deleteFilesInFolder(File folder) {
        File[] files = folder.listFiles();

        if (files != null) {
            for (File file : files) {
                if (file.isFile()) {
                    file.delete(); // 删除文件
                } else if (file.isDirectory()) {
                    deleteFilesInFolder(file); // 递归删除子文件夹下的文件
                }
            }
        }

        folder.delete(); // 删除空文件夹
    }

    /**
     * 本地文件转base64
     * @return
     */
    public static String imageToBase64(File file) {
        try {
            byte[] bytesArray = new byte[(int) file.length()];

            FileInputStream fileInputStream = new FileInputStream(file);
            fileInputStream.read(bytesArray);
            fileInputStream.close();
            return "data:image/jpeg;base64,"+Base64.getEncoder().encodeToString(bytesArray);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "";
    }

}

