package com.mortals.xhx.opencv.UI;

import com.mortals.xhx.opencv.utils.OpencvHelper;
import lombok.extern.slf4j.Slf4j;
import org.bytedeco.opencv.opencv_core.Mat;
import org.bytedeco.opencv.opencv_core.Rect;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;

/**
 * @author ZYW
 * @date 2024-01-02 10:26
 */
@Slf4j
public class VideoPanel extends JPanel {

    private Image image;

    private Rectangle rectangle;  //返回的矩形
    private Point drawStart, drawEnd;  //绘制起点、终点


    int cutType = 1; // 根据切边模式判断画线

    public VideoPanel() {
        // 初始化鼠标事件监听器
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                // 鼠标按下时记录起始点
                drawStart = new Point(e.getX(), e.getY());
                drawEnd = drawStart;
                // 触发重绘
                repaint();
            }
        });

        addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                // 鼠标拖动时更新终点
                drawEnd = new Point(e.getX(), e.getY());
                // 触发重绘
                repaint();
            }
        });

        addMouseListener(new MouseAdapter() {
            public void mouseReleased(MouseEvent e) {
                // 鼠标释放时完成绘制，并设置rectangle为最终矩形
                rectangle = new Rectangle(drawStart,
                        new Dimension(drawEnd.x - drawStart.x, drawEnd.y - drawStart.y));
                // 如果需要保持矩形不变，而不是继续绘制新的矩形，则取消注释下一行
                // removeMouseListener(this);
                // removeMouseMotionListener(this);
                // 触发重绘以显示完成的矩形（如果需要）
                repaint();
            }
        });
    }



    public void setImageWithMat(Mat mat, Rectangle angle,int cutType) {
        image = OpencvHelper.matToBufferedImage(mat);
        if(angle != null){
            this.rectangle = angle;
        }
        this.cutType = cutType;
        this.repaint();
    }

    public void setImage(BufferedImage bi,Rectangle angle) {
        image = bi;
        if(angle != null){
            this.rectangle = angle;
        }
        this.repaint();
    }



    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        //绘制图像
        if (image != null){
            g.drawImage(image, 0, 0, image.getWidth(null), image.getHeight(null), this);

        }

        Graphics2D g2d = (Graphics2D) g;
        // 设置画笔的粗细为5
        g2d.setStroke(new BasicStroke(2));

        g2d.setColor(Color.RED);
        // 如果正在绘制矩形，则使用临时的起点和终点来绘制它
        if (drawStart != null && drawEnd != null && cutType == 2) {
            g2d.drawRect(Math.min(drawStart.x, drawEnd.x), Math.min(drawStart.y, drawEnd.y),
                    Math.abs(drawEnd.x - drawStart.x), Math.abs(drawEnd.y - drawStart.y));
        }
        // 如果已经完成了矩形的绘制，则可以绘制一个填充的或未填充的矩形来表示它
        if (rectangle != null && cutType == 1) {
            g2d.drawRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        }
    }

    public Rect getRect() {
        Rect rect;
        if(rectangle != null){
            rect = new Rect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        }else {
            rect= null;
        }

        return rect; // 返回最终绘制的矩形，如果还没有完成绘制则返回null
    }


    public static VideoPanel show(String title, int width, int height, int open) {
        JFrame frame = new JFrame(title);
        if (open == 0) {
            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        } else {
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }

        frame.setSize(width, height);
        frame.setBounds(0, 0, width, height);
        VideoPanel videoPanel = new VideoPanel();
        videoPanel.setSize(width, height);
        frame.setContentPane(videoPanel);
        frame.setVisible(true);
        return videoPanel;
    }
}
