package com.mortals.xhx.opencv.utils;

import lombok.extern.slf4j.Slf4j;
import org.opencv.core.Mat;
import org.springframework.core.io.ClassPathResource;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @author ZYW
 * @date 2024-01-08 11:14
 */
@Slf4j
public class ImageUtil {

    /**
     * 图片纠斜
     * @param image
     * @return
     */
    public static BufferedImage skewCorrection(BufferedImage image) {
        // Convert image to grayscale
        BufferedImage grayscaleImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_BYTE_GRAY);
        Graphics g = grayscaleImage.getGraphics();
        g.drawImage(image, 0, 0, null);
        g.dispose();

        // Apply edge detection algorithm to find lines
        // ...

        // Determine angle of skew
        double skewAngle = 0.0;
        // ...

        // Rotate image to correct skew
        double radians = Math.toRadians(-skewAngle);
        double sin = Math.abs(Math.sin(radians));
        double cos = Math.abs(Math.cos(radians));
        int width = grayscaleImage.getWidth();
        int height = grayscaleImage.getHeight();
        int newWidth = (int) Math.floor(width * cos + height * sin);
        int newHeight = (int) Math.floor(height * cos + width * sin);
        BufferedImage rotatedImage = new BufferedImage(newWidth, newHeight, grayscaleImage.getType());
        Graphics2D g2d = rotatedImage.createGraphics();
        g2d.setColor(Color.WHITE);
        g2d.fillRect(0, 0, newWidth, newHeight);
        int x = (newWidth - width) / 2;
        int y = (newHeight - height) / 2;
        g2d.translate(x, y);
        g2d.rotate(radians, width / 2.0, height / 2.0);
        g2d.drawRenderedImage(grayscaleImage, null);
        g2d.dispose();

        return rotatedImage;
    }

    /**
     * 裁切黑边
     * @param image
     * @return
     */
    public static BufferedImage cutAuto(BufferedImage image) {
        BufferedImage originalImage = skewCorrection(image);

        // 获取图片的宽度和高度
        int width = originalImage.getWidth();
        int height = originalImage.getHeight();

        // 创建一个新的BufferedImage，其尺寸与原始图像相同
        BufferedImage imageWithBorder = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = imageWithBorder.createGraphics();
        g.drawImage(originalImage, 0, 0, null);
        g.dispose();

        // 遍历图像，查找黑色边界并裁剪
        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                int pixel = imageWithBorder.getRGB(x, y);
                int alpha = (pixel >> 24) & 0xff; // alpha通道
                if (alpha == 0) { // 如果像素是黑色的（alpha值为0）
                    // 裁剪黑色边界
                    BufferedImage croppedImage = new BufferedImage(width - x, height - y, BufferedImage.TYPE_INT_ARGB);
                    Graphics2D g2d = croppedImage.createGraphics();
                    g2d.drawImage(originalImage, 0, 0, x, y, null); // 只绘制被裁剪的部分
                    g2d.dispose();
                    imageWithBorder = croppedImage; // 将裁剪后的图像设置为当前图像
                    break; // 只需要找到一个黑色像素就停止遍历，因为整个图像都是黑色的了
                }
            }
        }

            return imageWithBorder;
    }

    /**
     * 读取项目目录下的图片文件
     */

    static public ImageIcon getImage(String filePath){
        ImageIcon imageIcon = null;
        ClassPathResource classPathResource = new ClassPathResource(filePath);
        try {
            imageIcon= new ImageIcon(ImageIO.read(classPathResource.getInputStream()));
        } catch (IOException e) {
            log.error("读取图片路径失败："+e.toString());
            throw new RuntimeException(e);
        }
        return imageIcon;
    }

    /**
     * 读取本地目录下的图片文件
     */
    static public ImageIcon getImageByFile(File file){
        ImageIcon imageIcon = null;
        try {
            imageIcon= new ImageIcon(ImageIO.read(file));
        } catch (IOException e) {
            log.error("读取图片路径失败："+e.toString());
            throw new RuntimeException(e);
        }
        return imageIcon;
    }

    /**
     * 读取本地目录下的图片文件 缩放成指定大小
     */
    static public ImageIcon zoomImageByFile(File file,int width,int height){
        ImageIcon imageIcon = null;
        try {
            // 读取图片
            BufferedImage originalImage = ImageIO.read(file);
            // 创建一个新的缓冲图片
            BufferedImage resizedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

            // 绘制并缩放原图到新图片
            Graphics2D g2d = resizedImage.createGraphics();
            g2d.drawImage(originalImage, 0, 0, width, width, null);
            g2d.dispose();

            imageIcon= new ImageIcon(resizedImage);
        } catch (IOException e) {
            log.error("读取图片路径失败："+e.toString());
            throw new RuntimeException(e);
        }
        return imageIcon;
    }
}
